//
// authour: Gregg Edghill
// Copyright (C) 2004, Gregg Edghill (gs_edghill@hotmail.com)
//
// button.cpp
//

#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <qcursor.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qtooltip.h>

#include "xpm/close.xpm"
#include "xpm/help.xpm"
#include "xpm/minimize.xpm"
#include "xpm/maximize.xpm"

#include "button.h"
#include "client.h"

namespace KDecorationWhid
{
	//
	// window border button constructor TODO
	//
	Button::Button(Client *parent, const char *name, const QString &tip, Qt::AlignmentFlags align, ButtonType type)
	: QButton(parent->widget(), name)
	, client (parent)
	{
		this->isMouseOver = false;
		this->isMaximized = false;
		this->state 	  = NoButton;
		this->type  	  = type;
		this->align 	  = align;

		setCursor(ArrowCursor);
		setBackgroundMode(NoBackground);

		if(type == menuButton)
			setFixedSize(16, 16);
		else if(align == AlignLeft || align == AlignRight)
			setFixedSize(27, 13);
		else
			setFixedSize(27, 13);

		QToolTip::add(this, tip);

// 		reset button icon
		resetIcon();
	}

	//
	// window border button destructor TODO
	//
	Button::~Button()
	{}

	//
	// return window border style client
	//
	Client* Button::GetClient()
	{
		return (client);
	}

	//
	// return mouse button state
	//
	Qt::ButtonState Button::mouseButtonState()
	{
		return (state);
	}

	//
	// reset the button
	//
	void Button::reset()
	{
		repaint(false);
	}

	//
	// indicate that the window border style client is maximized
	//
	void Button::setMaximized(bool maximized)
	{
		isMaximized = maximized;
		repaint (false);
	}

	//
	// return button size
	//
	QSize Button::sizeHint() const
	{
		return QSize(width(), height());
	}

	//
	// draw the window border button TODO
	//
	void Button::drawButton(QPainter *painter)
	{
		if(!Factory::Initialized())
		{
			return;
		}

		QColor titleColour  = client->options()->color(client->ColorTitleBar, client->isActive());
		QColor borderColour = titleColour.dark(200);
		QRect  button(0, 0, width(), height());
		QRect  region;

		if(type == menuButton)
		{
			QPixmap menuIcon = client->icon().pixmap(QIconSet::Small, QIconSet::Normal);
			int height = this->height();

			if(menuIcon.width() > height || menuIcon.height() > height)
				menuIcon.convertFromImage(menuIcon.convertToImage().smoothScale(height-2, height-2));

			region.setCoords(button.x(), button.y(), button.right(), button.bottom());
			painter->fillRect(region, titleColour);
			painter->drawPixmap((height-menuIcon.width())/2, (height-menuIcon.height())/2, menuIcon);
		}
		else
		{
// 			button gradient
			KPixmap gradient;
			gradient.resize(1, button.height() - 1);
			KPixmapEffect::gradient(gradient, titleColour.dark(100), (isMouseOver) ? titleColour.dark(300) : titleColour.dark(170), KPixmapEffect::VerticalGradient);

// 			button contours
			if(align == AlignLeft)
			{
				painter->drawTiledPixmap(button.left()+2, button.top()+1, button.width()-2, button.height()+2, gradient);

				painter->setPen(titleColour.light());
				painter->drawLine(button.left()+1, button.top(), button.right(), button.top());
				painter->drawLine(button.left()+1, button.top(), button.left()+1, button.bottom()-2);
				painter->drawPoint(button.left()+2, button.bottom()-2);
				painter->drawLine(button.right(), button.top(), button.right(), button.bottom()-1);
				painter->drawLine(button.left()+1, button.bottom()-1, button.right(), button.bottom()-1);

				painter->setPen(titleColour.dark(150));
				painter->drawPoint(button.left()+1, button.bottom()-1);
				painter->drawLine(button.left()+2, button.bottom(), button.right(), button.bottom());
				painter->drawLine(button.left(), button.top(), button.left(), button.bottom()-1);

				painter->setPen(titleColour);
				painter->drawPoint(button.left()+1, button.bottom());
				painter->drawLine(button.left(), button.bottom()-1, button.left(), button.bottom());
			}
			else if(align == AlignRight)
			{
				painter->drawTiledPixmap(button.left()+1, button.top()+1, button.width()-2, button.height()+2, gradient);

				painter->setPen(titleColour.light());
				painter->drawLine(button.left(), button.top(), button.right()-1, button.top());
				painter->drawLine(button.left(), button.top(), button.left(), button.bottom()-1);
				painter->drawLine(button.right()-1, button.top(), button.right()-1, button.bottom()-2);
				painter->drawPoint(button.right()-2, button.bottom()-2);
				painter->drawLine(button.left(), button.bottom()-1, button.right()-2, button.bottom()-1);

				painter->setPen(titleColour.dark(150));
				painter->drawPoint(button.right(), button.bottom()-1);
				painter->drawLine(button.left(), button.bottom(), button.right()-2, button.bottom());
				painter->drawLine(button.right(), button.top(), button.right(), button.bottom()-1);

				painter->setPen(titleColour);
				painter->drawPoint(button.right()-1, button.bottom());
				painter->drawLine(button.right(), button.bottom()-1, button.right(), button.bottom());
			}
			else
			{
				painter->drawTiledPixmap(button.left()+2, button.top()+1, button.width()-4, button.height()+2, gradient);

				painter->setPen(titleColour.light());
				painter->drawLine(button.left()+1, button.top(), button.right()-1, button.top());
				painter->drawLine(button.left()+1, button.top(), button.left()+1, button.bottom()-1);
				painter->drawLine(button.right()-1, button.top(), button.right()-1, button.bottom()-1);
				painter->drawLine(button.left()+1, button.bottom()-1, button.right()-1, button.bottom()-1);

				painter->setPen(titleColour.dark(150));
				painter->drawLine(button.left(), button.top(), button.left(), button.bottom());
				painter->drawLine(button.right(), button.top(), button.right(), button.bottom());
				painter->drawLine(button.left(), button.bottom(), button.right(), button.bottom());
			}

// 			button icon
			int ix = button.width()/2 - 4, iy = button.y()+2;

			if(isDown()) painter->drawImage(ix+1, iy+1, icon);
			else painter->drawImage(ix, iy, icon);
		}
	}

	//
	// reset button icons
	//
	void Button::resetIcon()
	{
		QImage ref;

		switch(type)
		{
			case minimizeButton:
				ref = QImage(minimizeIcon);
				break;

			case maximizeButton:
				ref = QImage(maximizeIcon);
				break;

			case closeButton:
				ref = QImage(closeIcon);
				break;

			case helpButton:
				ref = QImage(helpIcon);
				break;

			default:
				break;
		}

		icon = recolorImage(&ref, Qt::white).smoothScale(9, 8);

		repaint();
	}

	//
	// mouse over event handler
	//
	void Button::enterEvent(QEvent *event)
	{
		isMouseOver = true;

		repaint(false);
		QButton::enterEvent(event);
	}

	//
	// mouse leave event handler
	//
	void Button::leaveEvent(QEvent *event)
	{
		isMouseOver = false;

		repaint(false);
		QButton::leaveEvent(event);
	}

	//
	// button click event handler
	//
	void Button::mousePressEvent(QMouseEvent *mouse)
	{
		state = mouse->button();
		QMouseEvent mouseEvent(mouse->type(), mouse->pos(), mouse->globalPos(), mouse->button(), mouse->state());
		QButton::mousePressEvent(&mouseEvent);
	}

	//
	// button release event handler
	//
	void Button::mouseReleaseEvent(QMouseEvent *mouse)
	{
		state = mouse->button();
		QMouseEvent mouseEvent(mouse->type(), mouse->pos(), mouse->globalPos(), mouse->button(), mouse->state());
		QButton::mouseReleaseEvent(&mouseEvent);
	}
}

#include "button.moc"
