//
// authour: Gregg Edghill
// Copyright (C) 2004, Gregg Edghill (gs_edghill@hotmail.com)
//
// client.cpp
//

#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qtooltip.h>

#include "client.h"
#include "button.h"

namespace KDecorationWhid
{
	//
	// Padding for the window decoration
	//
	const int spacerItemPadding	 = 4;

	//
	// Supported window types
	//
	const int supportedWindowTypes = NET::NormalMask | NET::DesktopMask | NET::DockMask | NET::ToolbarMask | NET::MenuMask | NET::DialogMask | NET::OverrideMask | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask;

	//
	// Client window decoration constructor
	//
	Client::Client(KDecorationBridge *bridge, KDecorationFactory *factory)
	: KDecoration (bridge, factory)
	, components(0)
	{
		this->spacerItemTop          = 0;
		this->spacerItemTitle        = 0;
		this->spacerItemLeftOfTitle  = 0;
		this->spacerItemRightOfTitle = 0;
		this->spacerItemLeft         = 0;
		this->spacerItemRight        = 0;
		this->spacerItemBottom       = 0;

		this->buttonLayoutLeft       = 0;
		this->buttonLayoutRight      = 0;

		this->titleHeight  = 0;
		this->titleFont    = QFont();

		this->titleCaptionChanged = true;
		this->isWindowClosing = false;
	}

	//
	// Dispose of all window decoration components
	//
	Client::~Client()
	{
		for(int thisbutton=0; thisbutton < numberOfButtons; ++ thisbutton)
		if(buttonGroup [thisbutton])
		{
			delete buttonGroup [thisbutton];
		}
	}

	//
	// Initialize the window decoration layout
	//
	void Client::init()
	{
		createMainWidget(WNoAutoErase|WStaticContents);
		widget()->installEventFilter(this);
		widget()->setBackgroundMode(NoBackground);

		initializeComponents();

// 		createPixmapGradients();

		if(isToolWindow())
		{
			titleFont   = Factory::TitleFontTool();
			titleHeight = Factory::TitleHeightTool();
		}
		else
		{
			titleFont   = Factory::TitleFont();
			titleHeight = Factory::TitleHeight();
		}

		titleCaptionChanged = true;

		widget()->update(spacerItemTitle->geometry());
	}

	//
	// Update borders based on window states
	//
	void Client::borders(int& left, int& right, int& top, int& bottom) const
	{
		if((maximizeMode() == MaximizeFull))
		{
			left   = right = 0;
			bottom = (Factory::BorderSize() - 1);
			top    = titleHeight;

			spacerItemTop->changeSize(1, 2, QSizePolicy::Expanding, QSizePolicy::Fixed);
			spacerItemLeft->changeSize(left, 1, QSizePolicy::Fixed, QSizePolicy::Expanding);
			spacerItemLeftOfTitle->changeSize(2, titleHeight, QSizePolicy::Fixed, QSizePolicy::Fixed);
			spacerItemRight->changeSize(right, 1, QSizePolicy::Fixed, QSizePolicy::Expanding);
			spacerItemRightOfTitle->changeSize(4, titleHeight, QSizePolicy::Fixed, QSizePolicy::Fixed);
			spacerItemBottom->changeSize(1, bottom, QSizePolicy::Expanding, QSizePolicy::Fixed);
		}
		else
		{
			left = right = bottom = Factory::BorderSize();
     	  	top  = titleHeight + spacerItemPadding;

			spacerItemTop->changeSize(1, spacerItemPadding, QSizePolicy::Expanding, QSizePolicy::Fixed);
			spacerItemLeft->changeSize(left, 1, QSizePolicy::Fixed, QSizePolicy::Expanding);
			spacerItemLeftOfTitle->changeSize(spacerItemPadding+3, titleHeight, QSizePolicy::Fixed, QSizePolicy::Fixed);
			spacerItemRight->changeSize(right, 1, QSizePolicy::Fixed, QSizePolicy::Expanding);
			spacerItemRightOfTitle->changeSize(spacerItemPadding+3, titleHeight, QSizePolicy::Fixed, QSizePolicy::Fixed);
			spacerItemBottom->changeSize(1, bottom, QSizePolicy::Expanding, QSizePolicy::Fixed);
		}

//		activate updated layout
		widget()->layout()->activate();
	}

	//
	// Window decoration event handler
	//
	bool Client::eventFilter(QObject *obj, QEvent *event)
	{
		if(obj != widget())
		{
    		return false;
		}

		switch(event->type())
    	{
    		case QEvent::Resize:
        		resizeEvent();
        		return true;
    		case QEvent::Paint:
        		paintEvent(static_cast<QPaintEvent*>(event));
        		return true;
    		case QEvent::MouseButtonDblClick:
        		mouseDoubleClickEvent(static_cast<QMouseEvent*>(event));
        		return true;
    		case QEvent::MouseButtonPress:
        		processMousePressEvent(static_cast<QMouseEvent*>(event));
        		return true;
			case QEvent::Show:
				showEvent(static_cast<QShowEvent*>(event));
          		return true;
    		default:
        		break;
    	}

		return false;
	}

	//
	// Minimum size of a window
	//
	QSize Client::minimumSize() const
	{
		return widget()->minimumSize();
	}

	//
	// Resize the window decoration to QSize that
	//
	void Client::resize(const QSize& that)
	{
		widget()->resize(that);
	}

	//
	// Show the window decoration
	//
	void Client::show()
	{
		widget()->show();
	}

	//
	// handle window decoration title double click
	//
	void Client::mouseDoubleClickEvent(QMouseEvent *event)
	{
		if(spacerItemTitle->geometry().contains(event->pos()))
		{
			titlebarDblClickOperation();
		}
	}

	//
	// Paint the window decoration
	//
	void Client::paintEvent(QPaintEvent *event)
	{
		if(!Factory::Initialized())
		{
			return;
		}

// 		update title caption TODO
// 		if(titleCaptionChanged)
// 		{
// 			updateCaption();
// 		}

		QPainter painter(widget());

// 		window decoration regions
		QRect topRegion(spacerItemTop->geometry());
		QRect titleRegion(spacerItemTitle->geometry());
		QRect regionLeftOfTitle(spacerItemLeftOfTitle->geometry());
		QRect regionRightOfTitle(spacerItemRightOfTitle->geometry());
		QRect leftRegion(spacerItemLeft->geometry());
		QRect rightRegion(spacerItemRight->geometry());
		QRect bottomRegion(spacerItemBottom->geometry());

		QRect regionLeftButtons(buttonLayoutLeft->geometry());
		QRect regionRightButtons(buttonLayoutRight->geometry());

		QRect region;

// 		window decoration colours
		const QColor titleColour  = options()->color(ColorTitleBar, isActive());
		const QColor borderColour = titleColour.dark(300);

		//
		// top region
		//
		if(topRegion.height() > 0)
		{
			if(leftRegion.width() > 0)
			{
				painter.setPen(borderColour);
				painter.drawLine(topRegion.left()+4, topRegion.top(), topRegion.right()-4, topRegion.top());
				painter.drawLine(topRegion.left()+2, topRegion.top()+1, topRegion.left()+4, topRegion.top()+1);
				painter.drawLine(topRegion.right()-4, topRegion.top()+1, topRegion.right()-2, topRegion.top()+1);
				painter.drawLine(topRegion.left()+1, topRegion.top()+2, topRegion.left()+1, topRegion.top()+4);
				painter.drawLine(topRegion.right()-1, topRegion.top()+2, topRegion.right()-1, topRegion.top()+4);

				painter.setPen(titleColour.light());
				painter.drawLine(topRegion.left()+4, topRegion.top()+1, topRegion.right()-4, topRegion.top()+1);

				region.setCoords(topRegion.left()+2, topRegion.top()+2, topRegion.right()-2, topRegion.bottom());
				painter.fillRect(region, titleColour);
			}
			else
			if(topRegion.height() == 2)
			{
// 				window maximized
				painter.setPen(borderColour);
				painter.drawLine(topRegion.left(), topRegion.top(), topRegion.right(), topRegion.top());
				painter.drawPoint(topRegion.left(), topRegion.top()+1);
				painter.drawPoint(topRegion.right(), topRegion.top()+1);

				painter.setPen(titleColour.light());
				painter.drawLine(topRegion.left()+1, topRegion.top()+1, topRegion.right()-1, topRegion.top()+1);
			}
		}

		//
		// left title region
		//
		if(regionLeftOfTitle.width() > 0)
		{
			painter.setPen(borderColour);
			painter.drawLine(regionLeftOfTitle.left(), regionLeftOfTitle.top(), regionLeftOfTitle.left(), regionLeftOfTitle.bottom());

			if(leftRegion.width() > 0)
			{
				painter.setPen(titleColour.light());
				painter.drawLine(regionLeftOfTitle.left()+1, regionLeftOfTitle.top(), regionLeftOfTitle.left()+1, regionLeftOfTitle.bottom());

				region.setCoords(regionLeftOfTitle.left()+2, regionLeftOfTitle.top(), regionLeftOfTitle.right(), regionLeftOfTitle.bottom());
				painter.fillRect(region, titleColour);
			}
			else
			{
// 				window maximized

				region.setCoords(regionLeftOfTitle.left()+1, regionLeftOfTitle.top(), regionLeftOfTitle.right(), regionLeftOfTitle.bottom());
				painter.fillRect(region, titleColour);
			}
		}

		//
		// left button(s) region
		//
		region.setCoords(regionLeftButtons.left(), regionLeftButtons.top(), regionLeftButtons.right(), regionLeftButtons.bottom());
		painter.fillRect(region, titleColour);

		//
		// title region
		//
		if(titleRegion.width() > 0)
		{
			KPixmap titleGradient;
    		titleGradient.resize(1, titleRegion.height());
    		KPixmapEffect::gradient(titleGradient, titleColour, titleColour.light(100), KPixmapEffect::DiagonalGradient);
			painter.drawTiledPixmap(titleRegion.left(), titleRegion.top(), titleRegion.width(), titleRegion.height(), titleGradient);

			painter.setFont(titleFont);
			painter.setPen(options()->color(ColorFont, isActive()));

			QString text = caption();

			if(text.length() > titleRegion.width())
			{
				text.truncate(titleRegion.width() - 8);
				text.append(" ...");

				painter.drawText(titleRegion.left()+spacerItemPadding, titleRegion.top(), titleRegion.width()-spacerItemPadding , titleHeight-1, Factory::TitleFontAlignment(), text);
			}
			else
			{
				painter.drawText(titleRegion.left()+spacerItemPadding, titleRegion.top(), titleRegion.width()-spacerItemPadding , titleHeight-1, Factory::TitleFontAlignment(), text);
			}

		}

		//
		// right button(s) region
		//
		region.setCoords(regionRightButtons.left(), regionRightButtons.top(), regionRightButtons.right(), regionRightButtons.bottom());
		painter.fillRect(region, titleColour);

		//
		// right title region
		//
		if(regionRightOfTitle.width() > 0)
		{
			painter.setPen(borderColour);
			painter.drawLine(regionRightOfTitle.right(), regionRightOfTitle.top(), regionRightOfTitle.right(), regionRightOfTitle.bottom());

			if(rightRegion.width() > 0)
			{
				painter.setPen(titleColour.light());
				painter.drawLine(regionRightOfTitle.right()-1, regionRightOfTitle.top(), regionRightOfTitle.right()-1, regionRightOfTitle.bottom());

				region.setCoords(regionRightOfTitle.left(), regionRightOfTitle.top(), regionRightOfTitle.right()-2, regionRightOfTitle.bottom());
				painter.fillRect(region, titleColour);
			}
			else
			{
// 				window maximized
				region.setCoords(regionRightOfTitle.left(), regionRightOfTitle.top(), regionRightOfTitle.right()-1, regionRightOfTitle.bottom());
				painter.fillRect(region, titleColour);
			}
		}

		//
		// left and right regions
		//
		if(leftRegion.height() == 1 && rightRegion.height() == 1)
		{
// 			window is shaded
			painter.setPen(borderColour);
			painter.drawPoint(leftRegion.left(), leftRegion.top());
			painter.drawPoint(leftRegion.left(), leftRegion.top());
		}
		else
		{
			if(leftRegion.width() > 0)
			{
				region.setCoords(leftRegion.left()+2, leftRegion.top(), leftRegion.right(), leftRegion.bottom());
				painter.fillRect(region, titleColour);

				painter.setPen(titleColour.light());
				for(int y=leftRegion.top(); y <= leftRegion.bottom(); ++ y)
					painter.drawLine(leftRegion.left()+1, y, leftRegion.left()+1, y);

				painter.setPen(borderColour);
				painter.drawLine(leftRegion.left(), leftRegion.top(), leftRegion.left(), leftRegion.bottom());
			}

			if(rightRegion.width() > 0)
			{
				region.setCoords(rightRegion.left(), rightRegion.top(), rightRegion.right()-2, rightRegion.bottom());
				painter.fillRect(region, titleColour);

				painter.setPen(titleColour.light());
				for(int y=rightRegion.top(); y <= rightRegion.bottom(); ++ y)
					painter.drawLine(rightRegion.right()-1, y, rightRegion.right()-1, y);

				painter.setPen(borderColour);
				painter.drawLine(rightRegion.right(), rightRegion.top(), rightRegion.right(), rightRegion.bottom());
			}
		}

		//
		// bottom region
		//
		if(bottomRegion.height() > 0)
		{
			if(leftRegion.width() > 0)
			{
				painter.setPen(titleColour);
				painter.drawLine(bottomRegion.left()+2, bottomRegion.top(), bottomRegion.right()-2, bottomRegion.top());
				painter.drawLine(bottomRegion.left()+4, bottomRegion.bottom()-2, bottomRegion.right()-4, bottomRegion.bottom()-2);

				painter.setPen(titleColour.light());
				painter.drawPoint(bottomRegion.left()+1, bottomRegion.top());
				painter.drawPoint(bottomRegion.right()-1, bottomRegion.top());
				painter.drawLine(bottomRegion.left()+2, bottomRegion.bottom()-2, bottomRegion.left()+4, bottomRegion.bottom()-2);
				painter.drawLine(bottomRegion.right()-4, bottomRegion.bottom()-2, bottomRegion.right()-2, bottomRegion.bottom()-2);
				painter.drawLine(bottomRegion.left()+4, bottomRegion.bottom()-1, bottomRegion.right()-4, bottomRegion.bottom()-1);

				painter.setPen(borderColour);
				painter.drawLine(bottomRegion.left()+3, bottomRegion.bottom(), bottomRegion.right()-3, bottomRegion.bottom());
				painter.drawLine(bottomRegion.left()+1, bottomRegion.bottom()-1, bottomRegion.left()+3, bottomRegion.bottom()-1);
				painter.drawLine(bottomRegion.right()-3, bottomRegion.bottom()-1, bottomRegion.right()-1, bottomRegion.bottom()-1);
				painter.drawLine(bottomRegion.left(), bottomRegion.bottom()-2, bottomRegion.left()+1, bottomRegion.bottom()-2);
				painter.drawLine(bottomRegion.right()-1, bottomRegion.bottom()-2, bottomRegion.right(), bottomRegion.bottom()-2);
				painter.drawPoint(bottomRegion.left(), bottomRegion.top());
				painter.drawPoint(bottomRegion.right(), bottomRegion.top());
			}
			else
			{
// 				window maximized
				painter.setPen(titleColour);
				painter.drawLine(bottomRegion.left(), bottomRegion.top(), bottomRegion.right(), bottomRegion.top());
				painter.setPen(titleColour.light());
				painter.drawLine(bottomRegion.left(), bottomRegion.bottom()-1, bottomRegion.right(), bottomRegion.bottom()-1);
				painter.setPen(borderColour);
				painter.drawLine(bottomRegion.left(), bottomRegion.bottom(), bottomRegion.right(), bottomRegion.bottom());
			}
		}
	}

	//
	//
	//
	void Client::resizeEvent()
	{
		doShape();
    	widget()->update();
	}

	//
	//
	//
	void Client::showEvent(QShowEvent *event)
	{
		doShape();
    	widget()->update();
	}

	//
	//
	//
	void Client::windowWrapperShowEvent(QShowEvent *event)
	{
		doShape();
		widget()->update();
	}

	//
	// Active window focus has changed
	//
	void Client::activeChange()
	{
		for(int thisbutton=0; thisbutton < numberOfButtons; ++ thisbutton)
		if(buttonGroup [thisbutton])
		{
			buttonGroup [thisbutton]->repaint(false);
		}

		doShape();
		widget()->repaint(false);
	}

	//
	// Window title caption has changes
	//
	void Client::captionChange()
	{
		titleCaptionChanged = true;
		widget()->repaint(spacerItemTitle->geometry(), false);
	}

	//
	// current desktop has changed  TODO
	//
	void Client::desktopChange()
	{}

	//
	// Apply corner mask to the window decoration
	//
	void Client::doShape()
	{
		int w = widget()->width();
		int h = widget()->height();
		int right(w);
		int bottom(h);

		QRegion mask = QRegion(widget()->rect());

		if(spacerItemLeft->geometry().width() == 0)
		{
			mask = QRegion(widget()->rect());
			setMask(mask);
			return;
		}
		
		if(spacerItemTop->geometry().height() > 0)
		{
// 			top left corner mask
			if(spacerItemLeftOfTitle->geometry().width()  > 0)
			{
				mask -= QRegion(0, 0, 4, 1);
				mask -= QRegion(0, 1, 2, 1);
				mask -= QRegion(0, 2, 1, 2);
			}
// 			top right corner mask
			if(spacerItemRightOfTitle->geometry().width() > 0)
			{
				mask -= QRegion(right-4, 0, 4, 1);
				mask -= QRegion(right-2, 1, 2, 1);
				mask -= QRegion(right-1, 2, 1, 2);
			}
		}

		if(spacerItemBottom->geometry().height() > 0)
		{
// 			bottom left corner mask
			mask -= QRegion(0, bottom-1, 3, 1);
			mask -= QRegion(0, bottom-2, 1, 1);
// 			bottom right corner mask
			mask -= QRegion(right-3, bottom-1, 3, 1);
			mask -= QRegion(right-1, bottom-2, 1, 1);
		}

		setMask(mask);
	}

	//
	// the menu button icon has changed TODO
	//
	void Client::iconChange()
	{
		if(buttonGroup [menuButton])
		{
			buttonGroup [menuButton]->repaint(false);
		}
	}

	//
	// maximize state has change TODO
	//
	void Client::maximizeChange()
	{
		doShape();
		widget()->repaint(false);
	}

	//
	// Mouse Position relative to the window decoration
	//
	Client::Position Client::mousePosition(const QPoint &mouse) const
	{
		Position mousePosition = PositionCenter;
		const int corner       = 12;

// 		window border style regions
		QRect topRegion(spacerItemTop->geometry());
		QRect regionLeftOfTitle(spacerItemLeftOfTitle->geometry());
		QRect regionRightOfTitle(spacerItemRightOfTitle->geometry());
		QRect leftRegion(spacerItemLeft->geometry());
		QRect rightRegion(spacerItemRight->geometry());
		QRect bottomRegion(spacerItemBottom->geometry());

		if(topRegion.contains(mouse))
		{
			if(mouse.x() <= topRegion.left()+corner)		mousePosition = PositionTopLeft;
			else if(mouse.x() >= topRegion.right()-corner)	mousePosition = PositionTopRight;
			else											mousePosition = PositionTop;
		}
		else
		if(regionLeftOfTitle.contains(mouse))
		{
			if(mouse.y() <= topRegion.top()+corner)			mousePosition = PositionTopLeft;
			else											mousePosition = PositionLeft;
		}
		else
		if(regionRightOfTitle.contains(mouse))
		{
			if(mouse.y() <= topRegion.top()+corner)			mousePosition = PositionTopRight;
			else											mousePosition = PositionRight;
		}
		else
		if(leftRegion.contains(mouse))
		{
			if(mouse.y() <= topRegion.top()+corner)			mousePosition = PositionTopLeft;
			else if(mouse.y() >= bottomRegion.bottom()-corner)	mousePosition = PositionBottomLeft;
			else											mousePosition = PositionLeft;
		}
		else
		if(rightRegion.contains(mouse))
		{
			if(mouse.y() <= topRegion.top()+corner)			mousePosition = PositionTopRight;
			else if(mouse.y() >= bottomRegion.bottom()-corner)	mousePosition = PositionBottomRight;
			else											mousePosition = PositionRight;
		}
		else
		if(bottomRegion.contains(mouse))
		{
			if(mouse.x() <= bottomRegion.left()+corner)		mousePosition = PositionBottomLeft;
			else if(mouse.x() >= bottomRegion.right()-corner)	mousePosition = PositionBottomRight;
			else											mousePosition = PositionBottom;
		}

		return (mousePosition);
	}

	//
	// Handle specific changes to the window decoration
	//
	void Client::reset(unsigned long changed)
	{
		if(changed & SettingColors)
		{
			titleCaptionChanged = true;
			widget()->update();
		}
		else
		if(changed & SettingFont)
		{
			if(isToolWindow())
			{
				titleFont   = Factory::TitleFontTool();
				titleHeight = Factory::TitleHeightTool();
			}
			else
			{
				titleFont   = Factory::TitleFont();
				titleHeight = Factory::TitleHeight();
			}

			spacerItemTitle->changeSize(1, titleHeight, QSizePolicy::Expanding, QSizePolicy::Fixed);
			titleCaptionChanged = true;

			widget()->update();
		}
	}

	//
	// TODO
	//
    void Client::shadeChange()
	{
		doShape();
	}

	//
	// Determine whether the window type is a tool window
	//
	bool Client::isToolWindow()
	{
		NET::WindowType type = windowType(supportedWindowTypes);

		return ((type == NET::Toolbar) || (type == NET::Utility) || (type == NET::Menu));
	}

	//
	//
	//
	void Client::maximizeWindow()
	{
		if(buttonGroup[maximizeButton])
		switch(buttonGroup [maximizeButton]->mouseButtonState())
		{
			case LeftButton:
			default:
			{
				maximize((maximizeMode() == MaximizeFull) ? MaximizeRestore : MaximizeFull);
				maximizeChange();

				break;
			}
		}

		buttonGroup [maximizeButton]->setMaximized(maximizeMode() != MaximizeRestore);

		QToolTip::add(buttonGroup [maximizeButton], (maximizeMode() != MaximizeRestore) ? i18n("Restore") : i18n("Maximize"));
	}

	//
	// menu button press event handler
	//
	void Client::menuButtonPressed ()
	{
		if(buttonGroup [menuButton])
		{
			QPoint point(buttonGroup[menuButton]->rect().bottomLeft().x(), buttonGroup[menuButton]->rect().bottomLeft().y());
			showWindowMenu(buttonGroup[menuButton]->mapToGlobal(point));
		}
	}

	//
	//
	//
	void Client::menuButtonReleased()
	{
		if(isWindowClosing)
		{
			closeWindow();
		}
	}

	//
	// Update the title caption TODO
	//
	void Client::updateCaption()
	{}

	void Client::addButtons(QBoxLayout *parent, const QString &buttons)
	{
		if(buttons.length() > 0)
		{
			Qt::AlignmentFlags buttonAlign;
			for(unsigned thisbutton=0; thisbutton < buttons.length(); ++ thisbutton)
			{
				if((thisbutton == 0) && buttons.length() > 1) buttonAlign = AlignLeft;
				else if(thisbutton != 0 && (thisbutton != buttons.length() - 1)) buttonAlign = AlignHCenter;
				else buttonAlign = AlignRight;

				switch(buttons [thisbutton])
				{
					case 'M':
						if(!buttonGroup[menuButton])
						{
							buttonGroup[menuButton] = new Button(this, "menu", i18n("Menu"), buttonAlign, menuButton);
							connect(buttonGroup[menuButton], SIGNAL(pressed()), this, SLOT(menuButtonPressed()));
							connect(buttonGroup[menuButton], SIGNAL(released()), this, SLOT(menuButtonReleased()));

							parent->addWidget(buttonGroup[menuButton], 0, Qt::AlignHCenter|Qt::AlignTop);
						}
						break;

					case 'H':
						if(!buttonGroup[helpButton])
						{
							buttonGroup[helpButton] = new Button(this, "help", i18n("Help"), buttonAlign, helpButton);
							connect(buttonGroup[helpButton], SIGNAL (clicked()), this, SLOT (showHelpContents()));

							parent->addWidget(buttonGroup[helpButton], 0, Qt::AlignHCenter|Qt::AlignTop);
						}
						break;

					case 'I':
						if(!buttonGroup[minimizeButton])
						{
							buttonGroup[minimizeButton] = new Button(this, "minimize", i18n("Minimize"), buttonAlign, minimizeButton);
							connect(buttonGroup[minimizeButton], SIGNAL (clicked()), this, SLOT (minimize()));

							parent->addWidget(buttonGroup[minimizeButton], 0, Qt::AlignHCenter|Qt::AlignTop);
						}
						break;

					case 'A':
						if(!buttonGroup[maximizeButton] && isMaximizable())
						{
							const bool isMaximized = (maximizeMode() != MaximizeRestore);
							buttonGroup[maximizeButton] = new Button(this, "maximize", isMaximized ? i18n("Restore") : i18n("Maximize"),
																buttonAlign, maximizeButton);
							connect(buttonGroup[maximizeButton], SIGNAL (clicked()), this, SLOT (maximizeWindow()));

							parent->addWidget(buttonGroup[maximizeButton], 0, Qt::AlignHCenter|Qt::AlignTop);
						}
						break;

					case 'X':
						if(!buttonGroup[closeButton])
						{
							buttonGroup[closeButton] = new Button(this, "close", i18n("Close"), buttonAlign, closeButton);
							connect(buttonGroup[closeButton], SIGNAL (clicked()), this, SLOT (closeWindow()));

							parent->addWidget(buttonGroup[closeButton], 0, Qt::AlignHCenter|Qt::AlignTop);
						}
						break;

					case '_':
						parent->addSpacing(2);
						break;
				}

				if(thisbutton < (buttons.length() - 1))
				{
					parent->addSpacing(1);
				}
			}
		}
	}

	//
	// Reset the components of the window decoration layout to default
	//
	void Client::initializeComponents()
	{
		if(!Factory::Initialized())
		{
			return;
		}

		//
		// Delete all components
		//
		delete components;
		delete spacerItemTop;
		delete spacerItemTitle;
		delete spacerItemLeftOfTitle;
		delete spacerItemRightOfTitle;
		delete spacerItemLeft;
		delete spacerItemRight;
		delete spacerItemBottom;

		delete buttonLayoutLeft;
		delete buttonLayoutRight;

		//
		// Initialize all window decoration components
		//
		components   = new QVBoxLayout(widget(), 0, 0);

		spacerItemTop          = new QSpacerItem(1, spacerItemPadding, QSizePolicy::Expanding, QSizePolicy::Fixed);
		spacerItemTitle        = new QSpacerItem(1, titleHeight, QSizePolicy::Expanding, QSizePolicy::Fixed);
		spacerItemLeftOfTitle  = new QSpacerItem((spacerItemPadding + 2), titleHeight, QSizePolicy::Expanding, QSizePolicy::Fixed);
		spacerItemRightOfTitle = new QSpacerItem((spacerItemPadding + 2), titleHeight, QSizePolicy::Expanding, QSizePolicy::Fixed);
		spacerItemLeft         = new QSpacerItem(spacerItemPadding, 1, QSizePolicy::Expanding, QSizePolicy::Fixed);
		spacerItemRight        = new QSpacerItem(spacerItemPadding, 1, QSizePolicy::Expanding, QSizePolicy::Fixed);
		spacerItemBottom       = new QSpacerItem(1, spacerItemPadding, QSizePolicy::Expanding, QSizePolicy::Fixed);

		buttonLayoutLeft  = new QHBoxLayout();
		buttonLayoutRight = new QHBoxLayout();

		memset(buttonGroup, 0, sizeof(Button*)*numberOfButtons);

		//
		// Setup window decoration layout
		//
// 		 _____________________________________________________________________________
// 		|----------------------------- spacerItemTop ---------------------------------|
// 		|spacerItemLeftOfTitle | M |  spacerItemTitle  | HIAX | spacerItemRightOfTitle|
// 		|-----------------------------------------------------------------------------|
// 		||                                                                           ||
// 		||                                                                           ||
// 		||                                                                           ||
// 		||                                                                           ||
// 		||                                                                           ||
// 		|spacerItemLeft                                                spacerItemRight|
// 		||___________________________________________________________________________||
// 		'---------------------------- spacerItemBottom -------------------------------'

		//
		// top
		//
		components->addItem(spacerItemTop);

		//
		// title
		//
		QHBoxLayout *title   = new QHBoxLayout(components, 0, "title layout");

		title->addItem(spacerItemLeftOfTitle );
		addButtons(buttonLayoutLeft, options()->customButtonPositions() ? options()->titleButtonsLeft(): QString("M"));
		title->addItem(buttonLayoutLeft );
		title->addItem(spacerItemTitle);
		addButtons(buttonLayoutRight, options()->customButtonPositions() ? options()->titleButtonsRight(): QString("IAX"));
		title->addItem(buttonLayoutRight);
		title->addItem(spacerItemRightOfTitle);

		//
		// content
		//
		QHBoxLayout *content = new QHBoxLayout(components, 0, "content layout");
		content->addItem(Factory::IsLayoutReversed()? spacerItemRight : spacerItemLeft);
		if(this->isPreview()) content->addWidget(new QLabel(i18n( "<center><b>Whid preview</b></center>" ), widget()));
		else content->addItem(new QSpacerItem(0, 0));
		content->addItem(Factory::IsLayoutReversed() ? spacerItemLeft : spacerItemRight);

		//
		// bottom
		//
		components->addItem(spacerItemBottom);
	}
}

#include "client.moc"
